#include <petsc.h>
int main(int argc, char **args)
{
  PetscInitialize(&argc, &args, NULL, NULL);

  Vec b, x;
  Mat A;
  PetscViewer fd;

  /* Read matrix and vector from file*/

  /* Read matrix */
  PetscViewerBinaryOpen(PETSC_COMM_SELF, "mixed-poisson-mat-64.dat",
                        FILE_MODE_READ, &fd);
  MatCreate(PETSC_COMM_SELF, &A);
  MatSetType(A, MATSEQAIJ);
  MatLoad(A, fd);
  PetscViewerDestroy(&fd);

  /* Read vector */
  PetscViewerBinaryOpen(PETSC_COMM_SELF, "mixed-poisson-vec-64.dat",
                        FILE_MODE_READ, &fd);
  VecCreate(PETSC_COMM_SELF, &b);
  VecSetType(b, VECSEQ);
  VecLoad(b, fd);
  PetscViewerDestroy(&fd);

  /* Display matrix/vector */
  /*
  MatView(A, PETSC_VIEWER_STDOUT_WORLD);
  VecView(b, PETSC_VIEWER_STDOUT_WORLD);
  */

  /* Create solver (LU) */
  KSP ksp;
  KSPCreate(PETSC_COMM_SELF, &ksp);
  KSPSetType(ksp, KSPPREONLY);
  KSPSetOperators(ksp, A, A);

  PC pc;
  KSPGetPC(ksp, &pc);
  PCSetType(pc, PCLU);

  KSPSetFromOptions(ksp);

  /* Solve */
  VecDuplicate(b, &x);
  VecZeroEntries(x);
  KSPSolve(ksp, b, x);

  /* Check solution norm */
  PetscReal norm = 0.0;
  VecNorm(x, NORM_2, &norm);
  printf("Solution L2 norm: %f \n", norm);
  printf("Solution L2 norm should be: 2.96824 \n");

  PetscFinalize();
  return 0;
}
